/**
 * 功能：多篇文章刷取访问量
 * 参数:init(arr,time) arr要刷取访问量的主页地址，time多少时间刷一条
 */

const request = require("request");

const ProgressBar = require("../utils/progressBar");
const { GetUserInfo, emitter } = require("../utils/GetUserData");
const { outputProgressBar } = require("../utils/utils");

// 发起访问
/**
 *
 * @param {*} val [{title:"标题",url:"文章链接",viewCount:26}]
 * @param {*} time 每次访问间隔
 * @returns
 */
const visit = (val, time) => {
  if (val) {
    // 周期
    let count = 1;
    // 当前进度
    let completed = 0;
    // 总进度
    const total = val.length;

    // 已访问次数
    let visitTotal = 0;
    // 进度条实例
    const progress = new ProgressBar(`第${count}周期进度`, 50);

    // 开始时间 结束时间 总耗时
    let startTime = Date.now();
    let endTime = null;
    let s = null;

    try {
      setInterval(() => {
        visitTotal++;

        request(val[completed].url, (error, response, body) => {
          completed++;
          if (!error && response.statusCode === 200) {
            // 渲染进度条
            progress.render({
              completed,
              total,
            });

            // 完成一个周期
            if (completed == total - 1) {
              endTime = Date.now();
              s = (endTime - startTime) / 1000;
              startTime = Date.now();

              completed = 0;
              count++;

              progress.render({
                completed: total,
                total,
              });

              progress.editDes(`第${count}周期进度`);
              console.log(` √ 总耗时：${s}s`.blue);
            } else {
              console.log("visitTotal: ", visitTotal);
              let title =
                val[completed].title.length > 30
                  ? val[completed].title.slice(0, 30) + "..."
                  : val[completed].title;
              console.log(
                ` Total：${visitTotal}`,
                ` 标题：${title}`.red,
                `访问量:${val[completed].viewCount}+1`.green
              );
            }
          } else {
            // 失败
            console.log(
              `\n访问失败:Error=>${error}\n`.red,
              "请联系开发者(QQ:2845486124)".cyan
            );

            // 退出进程
            process.exit(1);
          }
        });
      }, time);
    } catch (error) {
      console.log(`error:${error}`.red);
    }
  } else {
    console.log("Error accessing, no links => visit function");
    console.log("访问失败,请联系开发者(QQ:2845486124)".cyan);
    // 退出进程
    // process.exit(1);
    // 递归访问
    visit(val, time);
  }
};

const singleVisit = (urlArr, time) => {
  // 总访问量
  let total = 0;
  // 索引
  let index = 0;
  // 开始时间
  let startTime = Date.now();
  let endTime = null;
  let s = null;

  try {
    setInterval(() => {
      request(urlArr[index], (error, response, body) => {
        endTime = Date.now();
        s = (endTime - startTime) / 1000;
        index++;
        if (index == urlArr.length) index = 0;

        if (!error && response.statusCode === 200) {
          total++;
          console.log(`Visit Total：${total}`.green, `Second：${s}s`.cyan);
        } else {
          // 失败
          console.log(
            `\n访问失败:Error=>${error}\n`.red,
            "请联系开发者(QQ:2845486124)".cyan
          );

          // 退出进程
          process.exit(1);
        }
      });
    }, time);
  } catch (error) {
    console.log("error: ".red, error);
    // 捕获错误递归访问
    singleVisit(urlArr, time);
  }
};

// 初始化
const init = async (config) => {
  // 进度条
  console.log("\n");
  await outputProgressBar("正在启动", "启动成功");

  // 开始计算启动时间，获取到链接信息结束 timeEnd
  console.time("启动总耗时");

  GetUserInfo([config.userID]);
  // 监听加载完成事件 接收数据
  emitter.on("loadListCompleted", (list) => {
    // 访问速度(条/ms)
    console.log("\n");
    console.timeEnd("启动总耗时");

    if (config.mode.type == "random") {
      list.sort(() => Math.random() - 0.5);
    } else if (config.mode.type == "first") {
      list = list.slice(0, config.mode.n);
    } else if (config.mode.type == "last") {
      list = list.reverse().slice(0, config.mode.n);
    }

    if (list.length < 10 && config.speed < 5000) {
      console.log("配速与文章数量不匹配，已调整为标准配速！".cyan);
      config.speed = 5000;
    } else if (list.length < 20 && config.speed < 2000) {
      console.log("配速与文章数量不匹配，已调整为标准配速！".cyan);
      config.speed = 2000;
    } else if (list.length < 40 && config.speed < 1000) {
      console.log("配速与文章数量不匹配，已调整为标准配速！".cyan);
      config.speed = 1000;
    }

    console.log(`当前速度(step/ms)：${config.speed}ms\n`.blue);

    visit(list, config.speed);
  });
};

const articleGroup = async (urlArr) => {
  await ProgressView();
  let time = 5000;
  if (urlArr.length < 3) {
    time = 20000;
    console.log(`当前速度${time / 1000}s一篇文章,请耐心等待`.cyan);
  } else if (urlArr.length < 6) {
    time = 10000;
    console.log(`当前速度${time / 1000}s一篇文章,请耐心等待`.cyan);
  }

  console.log(`当前速度(step/ms)：${time}ms\n`.blue);

  singleVisit(urlArr, time);
};

// 显示加载中信息
function ProgressView() {
  return new Promise((resolve) => {
    console.log("\n");
    const getProgress = new ProgressBar("正在启动", 50, false);
    let start = 1;
    let end = 30;
    let timer = setInterval(() => {
      start++;
      getProgress.render({
        completed: start,
        total: end,
      });
      if (start == 30) {
        getProgress.editDes("启动成功");
        getProgress.render(
          {
            completed: end,
            total: end,
          },
          "blue"
        );
        console.log(" √".blue);
        clearInterval(timer);
        resolve();
      }
    }, 10);
  });
}
module.exports = {
  init,
  articleGroup,
};
